package com.sodius.atl.xhtml;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;

import com.sodius.logging.MyLogger;

/**
 * The ValidXHTML class is used to validate a XHTML file generated by an ATL
 * transformation and using the default XML Extractor into a file conforms to
 * the W3C specifications.
 * 
 * @author &Eacute;ric V&eacute;pa
 * @version $Id: ValidXHTML.java 888 2007-08-13 15:48:16Z evepa $
 * @since 1.5
 */
public class ValidXHTML {

	/**
	 * Returns a usage string.
	 * 
	 * @return a usage string
	 */
	private static String usage() {
		return "Usage: java -jar com.sodius.atl.xhtml.jar inputFile\n"
				+ "  inputFile: XHTML file generated by an ATL transformation and\n"
				+ "             using the default XML Extractor\n";
	}

	/**
	 * Validates a file generated by an ATL transformation and using the default
	 * XML Extractor into a file conforms to the W3C specifications.
	 * 
	 * @param args
	 *            the input file generated by an ATL transformation and using
	 *            the default XML Extractor
	 */
	public static void main(String[] args) {
		if (args.length == 1) {
			String inputFile = args[0];
			MyLogger.info(ValidXHTML.class,
					"Validating XHTML file generated using ATL XML Extractor at: "
							+ inputFile);

			validateXHTML(inputFile);
		} else {
			System.out.println(usage());
			System.exit(1);
		}

		if (MyLogger.getStatus() == 1) {
			System.err.println("See log file for more details.");
			System.exit(MyLogger.getStatus());
		}
	}

	/**
	 * Validates a file generated by an ATL transformation and using the default
	 * XML Extractor into a file conforms to the W3C specifications.
	 * 
	 * @param inputFile
	 *            the file to format into valid XHTML file.
	 */
	static public void validateXHTML(String inputFile) {
		BufferedReader in = null;
		BufferedWriter out = null;
		String outputFile = inputFile + ".tmp";

		try {
			in = new BufferedReader(new FileReader(inputFile));
		} catch (FileNotFoundException fnfe) {
			MyLogger.warn(ValidXHTML.class, "File " + inputFile
					+ " can not be read");
		}

		try {
			out = new BufferedWriter(new FileWriter(outputFile));
		} catch (IOException ioe) {
			MyLogger.warn(ValidXHTML.class, "Temporary file " + outputFile
					+ " can not be created");
		}

		// Omitted the first line and add the XHTML valid first lines.
		String line;
		try {
			if ((line = in.readLine()) != null) {
				out
						.write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.1//EN\"");
				out.newLine();
				out
						.write("  \"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd\">");
				out.newLine();

				while ((line = in.readLine()) != null) {
					line = line.replace("&apos;", "'");
					line = line.replace("&quot;", "\"");
					line = line.replace("&lt;", "<");
					line = line.replace("&gt;", ">");
					line = line.replace("&amp;", "&");
					line = line.replace("&ne;", "!=");

					out.write(line);
					out.newLine();
				}
			}
		} catch (IOException e) {
			MyLogger.warn(ValidXHTML.class, "Error during the reading of "
					+ inputFile + " or the writing of " + outputFile);
		}

		try {
			out.close();
		} catch (IOException ioe) {
			MyLogger.warn(ValidXHTML.class, outputFile + " can not be closed");
		}

		try {
			in.close();
		} catch (IOException ioe) {
			MyLogger.warn(ValidXHTML.class, inputFile + " can not be closed");
		}

		renameFile(outputFile, inputFile, true);
	}

	/**
	 * Renames a file into other one.
	 * 
	 * @param oldFile
	 *            the old path of the file.
	 * @param newName
	 *            the new path of the file.
	 * @param override
	 *            if the new file already exists, it will be overridden
	 */
	static private void renameFile(String oldName, String newName,
			Boolean override) {
		File oldFile = new File(oldName);
		File newFile = new File(newName);

		if (override && newFile.exists())
			newFile.delete();

		if (!oldFile.renameTo(newFile))
			MyLogger.warn(ValidXHTML.class, "File " + oldName
					+ " can not be renamed in " + newName);
	}
}
